@@ -1,10 +1,7 @@
 type derivable =
-    DecOpt
-  | ArbitrarySizedSuchThat
-  | GenSizedSuchThatMonotonicOpt
-  | GenSizedSuchThatSizeMonotonicOpt
-  | GenSizedSuchThatCorrect
-  | SizedProofEqs
+  DecOpt
+| GenSizedSuchThat
+| EnumSizedSuchThat
 val derivable_to_string : derivable -> string
 val mk_instance_name : derivable -> string -> string
 val derive_dependent :
